//
// Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
// Ministerpräsidenten des Landes Schleswig-Holstein
// Staatskanzlei
// Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
//
// Lizenziert unter der EUPL, Version 1.2 oder - sobald
// diese von der Europäischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie dürfen dieses Werk ausschließlich gemäß
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
//
// https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
//
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
// ausdrücklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschränkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
//

package server

import (
	"bytes"
	"fachstellen-proxy/internal/mock"
	"github.com/stretchr/testify/assert"
	"net/http"
	"testing"
	"log"
    "github.com/sirupsen/logrus"
    "go.elastic.co/ecslogrus"
)

func TestRequestLoggingMiddleware(t *testing.T) {
    logger := logrus.New()
    logger.SetFormatter(&ecslogrus.Formatter{})
    log.SetOutput(logger.Writer())

	t.Run("should log request", func(t *testing.T) {
	    logger.SetLevel(logrus.DebugLevel)
		mock.SetUpGrpcServer()
		SetUpHttpGateway()

		var buf bytes.Buffer
		log.SetOutput(&buf)

		defer func() {
            log.SetOutput(nil)
        }()

		jsonData := []byte(`{"mukId": "testMukId"}`)
		http.Post("http://localhost:8080/api/fachstellen", "application/json", bytes.NewBuffer(jsonData))

		logOutput := buf.String()
		assert.Contains(t, "received POST request for /api/fachstellen with body: {\"mukId\": \"testMukId\"}", logOutput)
		assert.Contains(t, "successfully handled POST request for /api/fachstellen with body: {\"mukId\": \"testMukId\"}", logOutput)
	})

	t.Run("should not log request", func(t *testing.T) {
	    logger.SetLevel(logrus.InfoLevel)
		mock.SetUpGrpcServer()
		SetUpHttpGateway()

		var buf bytes.Buffer
		log.SetOutput(&buf)

		defer func() {
            log.SetOutput(nil)
        }()

		http.Get("http://localhost:8080/")

		logOutput := buf.String()
		assert.Empty(t, logOutput)
	})
}
